%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GoldsteinUnwrap2D implements 2D Goldstein branch cut phase unwrapping algorithm.
%
% References::
% 1. R. M. Goldstein, H. A. Zebken, and C. L. Werner, Satellite radar interferometry:
%    Two-dimensional phase unwrapping, Radio Sci., vol. 23, no. 4, pp. 713720, 1988.
% 2. D. C. Ghiglia and M. D. Pritt, Two-Dimensional Phase Unwrapping:
%    Theory, Algorithms and Software. New York: Wiley-Interscience, 1998.
%
% Inputs: 1. Complex image in .mat double format
%         2. Binary mask (optional)          
% Outputs: 1. Unwrapped phase image
%          2. Phase quality map
%
% This code can easily be extended for 3D phase unwrapping.
% Posted by Bruce Spottiswoode on 22 December 2008
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% %% REPLACE WITH YOUR IMAGES
% % IM = load('/home/ao/Documents/tugba/input/p2_128x128.csv');                         %Load complex image
% load 'Inputfield_58.mat';
% IM=field_apmask;
% % IM_mask=ones(size(IM));                     %Mask (if applicable)
% IM_mask=(IM~=0); 
% %%

function temp_IM=GoldsteinUnwrap2D(IM,IM_mask)

if ischar(IM)&&exist(IM,'file')
    IM2=load(IM);
    if isstruct(IM2)
        if isfield(IM2,'field_apmask')
            IM=IM2.field_apmask;
            IM_phase=angle(IM);
        elseif isfield(IM2,'phase_correction')
            IM=IM2.phase_correction;
            IM_phase=angle(IM);
        elseif isfield(IM2,'input')
            IM=IM2.input;
            IM_phase=IM;
        elseif isfield(IM2,'output')
            IM=IM2.output;
            IM_phase=IM;
        else
            disp('Error: File Structure not recognized');
            return
        end
    else
        IM=IM2;
        IM_phase=angle(IM);
    end
elseif isreal(IM)
    IM_phase=IM;
else
    IM_phase=angle(IM);
end

if nargin==1
    IM_mask=apmask_v2(size(IM,1),size(IM,1)); 
end
% % IM_mag=abs(IM);                             %Magnitude image
                         %Phase image
% % IM_phase = load('/home/ao/Documents/tugba/input/p2_128x128.csv');
% %IM_phase = load('/home/ao/Documents/tugba/input/p3_1024x1024.csv');

max_box_radius=4;                           %Maximum search box radius (pixels)
threshold_std=5;                            %Number of noise standard deviations used for thresholding the magnitude image

% figure;imagesc(IM_phase),colormap(gray)

residue_charge=PhaseResidues(IM_phase, IM_mask);                            %Calculate phase residues
branch_cuts=BranchCuts(residue_charge, max_box_radius, IM_mask);            %Place branch cuts
[IM_unwrapped, rowref, colref]=FloodFill(IM_phase, branch_cuts, IM_mask);   %Flood fill phase unwrapping
% figure;imagesc(IM_phase),colormap(gray)
% figure;imagesc(IM_unwrapped),colormap(gray)
% Display results
% figure; imagesc(residue_charge), colormap(gray), axis square, axis off, title('Phase residues (charged)');
% figure; imagesc(branch_cuts), colormap(gray), axis square, axis off, title('Branch cuts');
% %figure; imagesc(immultiply(IM_phase,IM_mask)), colormap(gray), axis square, axis off, title('Wrapped phase');
% imagesc(IM_phase.*double( ~branch_cuts)), colormap(gray), axis square, axis off, title('Phase image');
tempmin=min(min(IM_unwrapped));          %This bit is just done to create a pleasing display when a mask is used
temp=(IM_unwrapped==0);
temp_IM=IM_unwrapped;
temp_IM(temp)=tempmin;
temp_IM=temp_IM-min(temp_IM(:));
% figure; imagesc(temp_IM), colormap(gray), axis square, axis off, title('Unwrapped phase');

end

