function [a,phiRec,rms]=MatchZernike(phi,NZernike)
% function [a,phiRec,rms]=MatchZernike(phi,NZernike)
%
% Retrieve NZernike Zernike coefficients to the phase phi. The method does
% not contain the integral method to get a, but matches the reconstructed
% phase to the input phase by a least-square method
%
% Input:
% phi           Phase, masked parts (corners or central obscuration) are NaNs
% NZernike      The number of Zernike coefficeients to calculate
%
% Return:
% a             NZernike coefficients
% phiRec        The constructed wavefront from a
% rms           The rms value between phi and phiRec
% 26.11.2008 MK
%

phi(find(abs(phi)<1e-8))=NaN;
[x,y]=size(phi);
geo.N=x;

Z0=zeros(geo.N,geo.N,NZernike);
Z1=zeros(geo.N*geo.N,NZernike);

% Store the Zernike polynomials Z(r,phi) in a 3-D matrix and apply a mask
% for each layer
for i=1:NZernike,
    Z0(:,:,i)=zernimat(geo.N,i).*(~isnan(phi));
end

% Resort the matrix for a linear solution method
for x=1:geo.N,
    for y=1:geo.N,
        Z1(x+(y-1)*geo.N,1:NZernike)=Z0(x+(y-1)*geo.N+[0:NZernike-1]*geo.N^2);
    end
end

% Solve the linear system to match the Zernike reconstruction
%[U,S,V]=svds(Z1,rank(Z1));
[U,S,V]=svd(full(Z1));

warning('off','MATLAB:divideByZero');
Sinv=1./S';
warning('on','MATLAB:divideByZero');
Sinv(find(Sinv==inf))=0;

% Calculate the Zernike coefficients a
phiNaN=phi;
phi(isnan(phi))=0;
a=V*Sinv*U'*phi(:);

% Build the phase from the Zernike coeffiecients
phiRec=zeros(geo.N);
for i=1:NZernike,
    phiRec=phiRec+a(i)*zernimat(geo.N,i);
end;

%figure(13);
%imagesc(phiRec,[-7,7]); colorbar

% Calculate RMS of the residual match error
resid=(phiRec-phiNaN);
resMean=mean(mean(resid(find(~isnan(resid)))));

rms=sqrt(mean(mean((resid(find(~isnan(resid)))-resMean).^2)));


