function [r0P,r0] = r0_getstructf_Fried(compl_inputfield,dxa,nrfield,elevation,gflag,method)
% [r0P r0] = r0_getstructf_Fried(compl_inputfield,dxa,nrfield,elevation,gflag,method)
% 
% This function calculates r0 with the wave-structure function, the
% wave-structure-function D is the sum of the phase structure function and
% the normalized logamplitude-structure fuction 
%
% D(r) = 6.88*(r/r0).^(5/3) 
% --> D(r0) = 6.88
%
% Fried (Optical Heterodyne Detection of an Atmospherically distorted
% signal wavefront)
% 
% compl_inputfield      complex inputfield
% nrfield               Number of the field (for the figure title)
% elevation             Elevation angle of the field (for the figure title)
% dxa                   ratio between meter and pixel [m/pix]
% method                1: D(r0) = 6.88, 2: D(rho0) = 2, r0 = 2.1*rho0
% gflag                 1: graphics are displayed
%
% r0 is the Fried Parameter of the wave-structure function
% r0P is the Fried Parameter of the phase-structure function
% if r0 == -1: r0 > field (same for r0P)
% compares to the result of Extrapolation: D~rho^(5/3)
%
% MG, 2.Juli 2009
% last change: July 8, 2009
%
% needed functions: 
% getstructf -> xcf2, acf2, asf2, rotataverage
% rotataverage -> distmat, apmask
% distmat -> oddvec



if nargin == 5
    method = 1; 
end

if method == 1
    value = 6.88; 
else
    value = 2; 
end

phase = angle(compl_inputfield); 
amplitude = abs(compl_inputfield);
[structFuncP,rhoP] = getstructf_Nicotoolbox(phase,2,[],1);      % structure function of phase
rmsA = sqrt(sum(amplitude(:).^2));                  % root mean square of amplitude                
A = amplitude / rmsA;
A = log(A);                                         % logamplitude: A = log(A / rms(A))
[structFuncA,rhoA] = getstructf_Nicotoolbox(A,2,[],1);          % structure function of log-amplitude
rho = rhoA;                                         % rhoP == rhoA

structFunc = structFuncP + structFuncA;             % wave-structure-function

% -------------------------------- Calculate r0 with wave-structure fuction

if max(structFunc) >= value
    tmp = abs(structFunc(1:ceil(end/2)) - value);                       % find value of structure function nearest to 6.88
    [y,idx] = min(tmp);
    r0 = rho(idx)*dxa;
  
    if method == 2
        r0 = r0*2.1;       % r0 = rho0 * 2.1; 
    end

%     disp(['r0 (Phase + Amplitude): ' num2str(r0) ' m'])
else            % if the maximum value of the structure function is lower than 2 adv. 6.88
    r0 = -1; 
end



% getstructf: roundingflag == 1, function is faster and does not fluctuate,
% but method2 assumes gaussian statistics. If there are branchpoints in the
% field, the field won't be gaussian anymore.
% For method 1 and method 3 you have to unwrap the field. 

% -------------------------------calculate r0 with phase-structure function

if max(structFuncP) >= value

    tmpP = abs(structFuncP(1:ceil(end/2)) - value);     
    % maybe the value is reached two times by the function, but the value
    % in the first half should be used, not the second part where the
    % structure function is decreasing. (Because the structure function
    % should be everywhere increasing, so the values in the decreasing
    % part are wrong). 

    [yP,idxP] = min(tmpP);      % find the value next to 6.88 adv. 2
    r0P = rhoP(idxP)*dxa; 

    if method == 2
        r0P = r0P * 2.1; 
    end
    
%     disp(['r0 (Phase): ' num2str(r0P) ' m'])
else
    r0P = -1;
end

%------------------------------------- Calculation of r0 with Extrapolation
%------------------------------------- For Comparsion

min_part_for_extrap = 0.05;             
% [m] for Extrapolation there are at least the first min_part_for_extrap m of the structure function used

% [peaks,locs] = findpeaks(structFunc); 

%%% For Extrapolation: Use the values before the first Peak, because after
%%% the peak the function starts to fluctuate
%%% then find a turning point in the part before the first Peak, take the
%%% values before the turning point. Because at a turning point, a function
%%% should change its behavior, then the Extrapolation doesn't fit.

structFunc2 = structFunc(1:end); 

tmp = abs(diff(structFunc2));           % Maximum of the first derivate -> turning point
[y,idx] = max(tmp); 

if rho(idx)*dxa < min_part_for_extrap
    % Use values before the first peak but at least the first 5 cms, if the
    % is a problem with the peaks. 
%     idx = locs(1);
    idx = 1; 
end

rho2 = rho(1:idx);
structFunc2 = structFunc(1:idx); 

A = rho2.^(5/3); 
factor = A\structFunc2;             % least square solution for : A*factor = structFunc2
ynew = factor*rho.^(5/3); 
res = A*factor-structFunc2;         % residuum >> error of least square solution
res = norm(res); 

% disp([num2str(nrfield) ' ' num2str(elevation) ' ,res - Extrapolation: ' num2str(res)])

tmp = abs(ynew-value);              % find the value next to 6.88 adv. 2
[yr0,idxr0] = min(tmp); 

% if method == 1
%     disp([num2str(nrfield) ' ' num2str(elevation) ', yr0 (should be 6.88): ' num2str(ynew(idxr0))])
% elseif method == 2
%     disp([num2str(nrfield) ' ' num2str(elevation) ', yr0 (should be 2): ' num2str(ynew(idxr0))])
% end

r0extrap = rho(idxr0)*dxa; 

% disp([ num2str(nrfield) ' ' num2str(elevation) ' ,r0 - Extrapolation: ' num2str(r0extrap)]);
                
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% graphic        
        
        
if gflag

    figure(456)
    title(['Field' num2str(nrfield) ', ' num2str(elevation) ', getstructf(methode2,roundingflag)'],'Fontsize',14)
    line(rhoA*dxa,structFuncA)
    line(rhoP*dxa,structFuncP,'Color','magenta')
    line(rho*dxa,structFunc,'Color','green')
    legend('SF norm. logamplitude [ ]','SF phase [rad]','SF norm. logamplitude + phase [rad]')
    xlabel('Distance [m]','Fontsize',14);
    ylabel(sprintf('Phase-Structure-Function [rad]  |\n norm. Logamplitude-Structure Function [  ]'),'Fontsize',14);    
    
    line(r0,6.88,'LineStyle','none','Marker','x','Color','red','LineWidth',1.5)
    
    if method == 2
        line(r0/2.1,2,'LineStyle','none','Marker','x','Color','black','LineWidth',1.5)
    end
    
    line(rho*dxa,ynew,'Color',[1 0.5 0])            % extrapolation (to compare)
    
    if method == 2
        legend('SF norm. logamplitude [ ]','SF phase [rad]','SF norm. logamplitude + phase [rad]','r_0','\rho_0','D~\rho^{5/3} Extrapolation')
    else
        legend('SF norm. logamplitude [ ]','SF phase [rad]','SF norm. logamplitude + phase [rad]','r_0','D~\rho^{5/3} Extrapolation')
    end
    
    % set(gca,'Fontsize',14)
end



